; CircuitBread Tutorials
; Microcontroller Basics (PIC10F200), Part 8 - Musical Microcontroller
; Link: https://www.circuitbread.com/tutorials/musical-microcontroller-part-8-microcontroller-basics-pic10f200
; Source code conversion from Microchip MPASM to Microchip XC8 PIC Assembler.

; Include target device specific definitions.
#include <xc.inc>

; Some equates not included in the device specific definitions.
;----- OPTION_REG Bits -----------------------------------------------------
T0CS EQU 0005h
;----- STATUS Bits -----------------------------------------------------
Z    EQU 0002h

; Set the configuration word.
CONFIG WDTE = OFF  ; Watchdog Timer (WDT disabled)
CONFIG CP = OFF    ; Code Protect (Code protection off)
CONFIG MCLRE = ON  ; Master Clear Enable (MCLR enabled, GP3 disabled)

d1      EQU 010h    ;define 0x10 register as lower delay byte
d2      EQU 011h    ;define 0x11 register as upper delay byte
periods EQU 012h    ;define 0x12 register as number of periods to play

; NOTE: To make sure the PIC10F200 RC oscillator calibration instruction at
; the 0xFF program memory address is not overwriten place this psect elsewhere
; , for example at the program memory start address: Project Properties >
; pic-as Global Options > Additional Options: -Wl,-pMyCode=0h
    PSECT MyCode,class=CODE,delta=2
INIT:
    MOVLW  ~(1<<T0CS)               ;enable GPIO2
    OPTION
    MOVLW ~(1<<GPIO_GP2_POSITION)   ;set GP2 as an output
    TRIS GPIO
LOOP:
    CALL E2             ;Play note E of the 2nd octave
    CALL D_2            ;Play note D# of the 2nd octave
    CALL E2             ;and so on and so forth
    CALL D_2
    CALL E2
    CALL H1
    CALL D2
    CALL C2
    CALL A1
    CALL A1
    CALL A1
    CALL C1
    CALL E1
    CALL A1
    CALL H1
    CALL H1
    CALL H1
    CALL E1
    CALL G_1
    CALL H1
    CALL C2
    CALL C2
    CALL C2
    SLEEP                ;Enable sleep mode
    GOTO LOOP            ;loop forever

E2:                      ;Note E of the 2nd octave
    MOVLW 255            ;Load the number of periods to play
    MOVWF periods
LOOP_E2:                 ;Toggle pin GP2 with the specified frequency
    MOVLW (1<<GPIO_GP2_POSITION)
    XORWF GPIO, F        ;Toggle GP2
    MOVLW 251
    MOVWF d1             ;Load lower delay byte
    MOVLW 1
    MOVWF d2             ;Load upper delay byte
    CALL DELAY           ;Perform delay
    DECFSZ periods, F    ;Decrease the number of periods and check if it is 0
    GOTO LOOP_E2         ;If no then keep toggling GP2
    RETLW 0              ;Otherwise return from the subroutine

D_2:
    MOVLW 240
    MOVWF periods
LOOP_D_2:
    MOVLW (1<<GPIO_GP2_POSITION)
    XORWF GPIO, F
    MOVLW 10
    MOVWF d1
    MOVLW 2
    MOVWF d2
    CALL DELAY
    DECFSZ periods, F
    GOTO LOOP_D_2
    RETLW 0

H1:
    MOVLW 191
    MOVWF periods
LOOP_H1:
    MOVLW (1<<GPIO_GP2_POSITION)
    XORWF GPIO, F
    MOVLW 80
    MOVWF d1
    MOVLW 2
    MOVWF d2
    CALL DELAY
    DECFSZ periods, F
    GOTO LOOP_H1
    RETLW 0

D2:
    MOVLW 227
    MOVWF periods
LOOP_D2:
    MOVLW (1<<GPIO_GP2_POSITION)
    XORWF GPIO, F
    MOVLW 26
    MOVWF d1
    MOVLW 2
    MOVWF d2
    CALL DELAY
    DECFSZ periods, F
    GOTO LOOP_D2
    RETLW 0

C2:
    MOVLW 202
    MOVWF periods
LOOP_C2:
    MOVLW (1<<GPIO_GP2_POSITION)
    XORWF GPIO, F
    MOVLW 61
    MOVWF d1
    MOVLW 2
    MOVWF d2
    CALL DELAY
    DECFSZ periods, F
    GOTO LOOP_C2
    RETLW 0

A1:
    MOVLW 170
    MOVWF periods
LOOP_A1:
    MOVLW (1<<GPIO_GP2_POSITION)
    XORWF GPIO, F
    MOVLW 121
    MOVWF d1
    MOVLW 2
    MOVWF d2
    CALL DELAY
    DECFSZ periods, F
    GOTO LOOP_A1
    RETLW 0

C1:
    MOVLW 101
    MOVWF periods
LOOP_C1:
    MOVLW (1<<GPIO_GP2_POSITION)
    XORWF GPIO, F
    MOVLW 123
    MOVWF d1
    MOVLW 3
    MOVWF d2
    CALL DELAY
    DECFSZ periods, F
    GOTO LOOP_C1
    RETLW 0

E1:
    MOVLW 127
    MOVWF periods
LOOP_E1:
    MOVLW (1<<GPIO_GP2_POSITION)
    XORWF GPIO, F      	 
    MOVLW 248
    MOVWF d1
    MOVLW 2
    MOVWF d2
    CALL DELAY
    DECFSZ periods, F
    GOTO LOOP_E1
    RETLW 0

G_1:
    MOVLW 160
    MOVWF periods
LOOP_G_1:
    MOVLW (1<<GPIO_GP2_POSITION)
    XORWF GPIO, F
    MOVLW 144
    MOVWF d1
    MOVLW 2
    MOVWF d2
    CALL DELAY
    DECFSZ periods, F
    GOTO LOOP_G_1
    RETLW 0

DELAY:              ;Start DELAY subroutine here
    DECFSZ d1, F    ;Decrement the register 0x10 and check if not zero
    GOTO DELAY      ;If not then go to the DELAY_LOOP label
    DECFSZ d2, F    ;Else decrement the register 0x11, check if it is not 0
    GOTO DELAY      ;If not then go to the DELAY_LOOP label
    RETLW 0         ;Else return from the subroutine

    END INIT        ; Program entry point.